@@ -1,6 +1,8 @@
 /*
  * hash.c: support functions for tb_hash_t type
  *
+ * Copyright (c) 2020 Cisco Systems, Inc. <pmoore2@cisco.com>
+ *
  * Copyright (c) 2014, Intel Corporation
  * All rights reserved.
  *
@@ -39,9 +41,10 @@
 #include <stdbool.h>
 #include <string.h>
 #include <openssl/evp.h>
+#include <safe_lib.h>
 #define PRINT   printf
-#include "../include/config.h"
-#include "../include/hash.h"
+#include "../../include/config.h"
+#include "../../include/hash.h"
 
 /*
  * are_hashes_equal
@@ -52,19 +55,20 @@
 bool are_hashes_equal(const tb_hash_t *hash1, const tb_hash_t *hash2,
 		      uint16_t hash_alg)
 {
+    int diff;
     if ( ( hash1 == NULL ) || ( hash2 == NULL ) )
         return false;
 
     if ( hash_alg == TB_HALG_SHA1 )
-        return (memcmp(hash1, hash2, SHA1_LENGTH) == 0);
+        return (memcmp_s(hash1, SHA1_LENGTH, hash2, SHA1_LENGTH, &diff) == 0 && diff == 0);
     else if ( hash_alg == TB_HALG_SHA256 )
-        return (memcmp(hash1, hash2, SHA256_LENGTH) == 0);
+        return (memcmp_s(hash1, SHA256_LENGTH, hash2, SHA256_LENGTH, &diff) == 0 && diff == 0);
     else if ( hash_alg == TB_HALG_SM3 )
-        return (memcmp(hash1, hash2, SM3_LENGTH) == 0);
+        return (memcmp_s(hash1, SM3_LENGTH, hash2, SM3_LENGTH, &diff) == 0 && diff == 0);
     else if ( hash_alg == TB_HALG_SHA384 )
-        return (memcmp(hash1, hash2, SHA384_LENGTH) == 0);
+        return (memcmp_s(hash1, SHA384_LENGTH, hash2, SHA384_LENGTH, &diff) == 0 && diff == 0);
     else if ( hash_alg == TB_HALG_SHA512 )
-        return (memcmp(hash1, hash2, SHA512_LENGTH) == 0);
+        return (memcmp_s(hash1, SHA512_LENGTH, hash2, SHA512_LENGTH, &diff) == 0 && diff == 0);
     else
         return false;
 }
@@ -81,7 +85,7 @@ bool hash_buffer(const unsigned char* buf, size_t size, tb_hash_t *hash,
     if ( hash == NULL )
         return false;
 
-    if ( hash_alg == TB_HALG_SHA1 ) {
+    if ( hash_alg == TB_HALG_SHA1 || hash_alg == TB_HALG_SHA1_LG ) {
         EVP_MD_CTX *ctx = EVP_MD_CTX_create();
         const EVP_MD *md;
 
@@ -114,6 +118,27 @@ bool hash_buffer(const unsigned char* buf, size_t size, tb_hash_t *hash,
         EVP_MD_CTX_destroy(ctx);
         return true;
     }
+    else if (hash_alg == TB_HALG_SHA512) {
+        EVP_MD_CTX *ctx = EVP_MD_CTX_create();
+        const EVP_MD *md;
+
+        md = EVP_sha512();
+        EVP_DigestInit(ctx, md);
+        EVP_DigestUpdate(ctx, buf, size);
+        EVP_DigestFinal(ctx, hash->sha512, NULL);
+        EVP_MD_CTX_destroy(ctx);
+        return true;
+    }
+    else if (hash_alg == TB_HALG_SM3) {
+        EVP_MD_CTX *ctx = EVP_MD_CTX_create();
+        const EVP_MD *md;
+        md = EVP_sm3();
+        EVP_DigestInit(ctx, md);
+        EVP_DigestUpdate(ctx, buf, size);
+        EVP_DigestFinal(ctx, hash->sm3, NULL);
+        EVP_MD_CTX_destroy(ctx);
+        return true;
+    }
     else
         return false;
 }
@@ -135,8 +160,9 @@ bool extend_hash(tb_hash_t *hash1, const tb_hash_t *hash2, uint16_t hash_alg)
         EVP_MD_CTX *ctx = EVP_MD_CTX_create();
         const EVP_MD *md;
 
-        memcpy(buf, &(hash1->sha1), sizeof(hash1->sha1));
-        memcpy(buf + sizeof(hash1->sha1), &(hash2->sha1), sizeof(hash1->sha1));
+        memcpy_s(buf, sizeof(buf), &(hash1->sha1), sizeof(hash1->sha1));
+        memcpy_s(buf + sizeof(hash1->sha1), sizeof(buf) - sizeof(hash1->sha1),
+                 &(hash2->sha1), sizeof(hash1->sha1));
         md = EVP_sha1();
         EVP_DigestInit(ctx, md);
         EVP_DigestUpdate(ctx, buf, 2*sizeof(hash1->sha1));
@@ -169,10 +195,56 @@ void print_hash(const tb_hash_t *hash, uint16_t hash_alg)
         }
         printf("\n");
     }
+    else if ( hash_alg == TB_HALG_SHA384 ) {
+        for ( unsigned int i = 0; i < SHA384_LENGTH; i++ ) {
+            printf("%02x", hash->sha384[i]);
+            if ( i < SHA384_LENGTH-1 )
+                printf(" ");
+        }
+        printf("\n");
+    }
+    else if ( hash_alg == TB_HALG_SHA512 ) {
+        for ( unsigned int i = 0; i < SHA512_LENGTH; i++ ) {
+            printf("%02x", hash->sha512[i]);
+            if ( i < SHA512_LENGTH-1 )
+                printf(" ");
+        }
+        printf("\n");
+    }
+    else if ( hash_alg == TB_HALG_SM3) {
+        for ( unsigned int i = 0; i < SM3_LENGTH; i++ ) {
+            printf("%02x", hash->sm3[i]);
+            if ( i < SM3_LENGTH-1 )
+                printf(" ");
+        }
+        printf("\n");
+    }
     else
         return;
 }
 
+/*
+ * import a hash in the format "755567de6e0a3ee1b71a895b76..."
+ */
+bool import_hash(const char *string, tb_hash_t *hash, uint16_t alg)
+{
+    size_t hash_len = get_hash_size(alg);
+    size_t string_len = strnlen(string, hash_len * 2 + 1);
+    unsigned int iter_a, iter_b;
+    char byte[3] = {0, 0, 0};
+
+    for (iter_a = 0, iter_b = 0;
+         iter_a < string_len && iter_b <= hash_len;
+         iter_a += 2, iter_b++) {
+        memcpy_s(byte, sizeof(byte), &string[iter_a], 2);
+        hash->sha512[iter_b] = strtol(byte, NULL, 16);
+    }
+    if (iter_a != string_len || iter_b != hash_len)
+        return false;
+
+    return true;
+}
+
 void copy_hash(tb_hash_t *dest_hash, const tb_hash_t *src_hash,
                uint16_t hash_alg)
 {
@@ -180,7 +252,7 @@ void copy_hash(tb_hash_t *dest_hash, const tb_hash_t *src_hash,
         return;
 
     if ( hash_alg == TB_HALG_SHA1 )
-        memcpy(dest_hash, src_hash, SHA1_LENGTH);
+        memcpy_s(dest_hash, SHA1_LENGTH, src_hash, SHA1_LENGTH);
     else
         return;
 }
