@@ -38,14 +38,17 @@
 #include <stdbool.h>
 #include <printk.h>
 #include <cmdline.h>
-#include <multiboot.h>
+#include <string.h>
+#include <uuid.h>
+#include <loader.h>
 #include <stdarg.h>
 #include <misc.h>
 #include <pci_cfgreg.h>
+#include <efi_memmap.h>
 #include <e820.h>
 #include <txt/config_regs.h>
 
-/* minimum size of RAM (type 1) region that cannot be marked as resreved even
+/* minimum size of RAM (type 1) region that cannot be marked as reserved even
    if it comes after a reserved region; 0 for no minimum (i.e. current
    behavior) */
 uint32_t g_min_ram = 0;
@@ -58,11 +61,10 @@ uint32_t g_min_ram = 0;
 static unsigned int g_nr_map;
 static memory_map_t *g_copy_e820_map = (memory_map_t *)TBOOT_E820_COPY_ADDR;
 
-static inline void split64b(uint64_t val, uint32_t *val_lo, uint32_t *val_hi)
-{
-    *val_lo = (uint32_t)(val & 0xffffffff);
-    *val_hi = (uint32_t)(val >> 32);
-}
+static inline void split64b(uint64_t val, uint32_t *val_lo, uint32_t *val_hi)  {
+     *val_lo = (uint32_t)(val & 0xffffffff); 
+     *val_hi = (uint32_t)(val >> 32);
+ }
 
 static inline uint64_t combine64b(uint32_t val_lo, uint32_t val_hi)
 {
@@ -95,7 +97,7 @@ static void print_map(memory_map_t *e820, int nr_map)
         base_addr = e820_base_64(entry);
         length = e820_length_64(entry);
 
-        printk("\t%016Lx - %016Lx  (%d)\n",
+        printk(TBOOT_DETA"\t%016Lx - %016Lx  (%d)\n",
                (unsigned long long)base_addr,
                (unsigned long long)(base_addr + length),
                entry->type);
@@ -267,6 +269,17 @@ static bool is_overlapped(uint64_t base, uint64_t end, uint64_t e820_base,
     return false;
 }
 
+/* helper funcs for loader.c */
+memory_map_t *get_e820_copy()
+{
+    return g_copy_e820_map;
+}
+
+unsigned int get_nr_map()
+{
+    return g_nr_map;
+}
+
 /*
  * copy_e820_map
  *
@@ -274,23 +287,24 @@ static bool is_overlapped(uint64_t base, uint64_t end, uint64_t e820_base,
  *
  * return:  false = error (no table or table too big for new space)
  */
-bool copy_e820_map(const multiboot_info_t *mbi)
+bool copy_e820_map(loader_ctx *lctx)
 {
     get_tboot_min_ram();
 
     g_nr_map = 0;
 
-    if ( mbi->flags & MBI_MEMMAP ) {
-        printk("original e820 map:\n");
-        print_map((memory_map_t *)mbi->mmap_addr,
-                  mbi->mmap_length/sizeof(memory_map_t));
+    if (have_loader_memmap(lctx)){
+        uint32_t memmap_length = get_loader_memmap_length(lctx);
+        memory_map_t *memmap = get_loader_memmap(lctx);
+        printk(TBOOT_DETA"original e820 map:\n");
+        print_map(memmap, memmap_length/sizeof(memory_map_t));
 
         uint32_t entry_offset = 0;
 
-        while ( entry_offset < mbi->mmap_length &&
+        while ( entry_offset < memmap_length &&
                 g_nr_map < MAX_E820_ENTRIES ) {
             memory_map_t *entry = (memory_map_t *)
-                                       (mbi->mmap_addr + entry_offset);
+                (((uint32_t) memmap) + entry_offset);
 
             /* we want to support unordered and/or overlapping entries */
             /* so use protect_region() to insert into existing map, since */
@@ -299,21 +313,30 @@ bool copy_e820_map(const multiboot_info_t *mbi)
                                  e820_base_64(entry), e820_length_64(entry),
                                  entry->type) )
                 return false;
-            entry_offset += entry->size + sizeof(entry->size);
+            if (lctx->type == 1)
+                entry_offset += entry->size + sizeof(entry->size);
+            if (lctx->type == 2)
+                /* the MB2 memory map entries don't have a size--
+                 * they have a "zero" with a value of zero. Additionally,
+                 * because they *end* with a size and the MB1 guys *start*
+                 * with a size, we get into trouble if we try to use them,
+                 */
+                entry_offset += sizeof(memory_map_t);
+
         }
         if ( g_nr_map == MAX_E820_ENTRIES ) {
-            printk("Too many e820 entries\n");
+            printk(TBOOT_ERR"Too many e820 entries\n");
             return false;
         }
     }
-    else if ( mbi->flags & MBI_MEMLIMITS ) {
-        printk("no e820 map, mem_lower=%x, mem_upper=%x\n",
-               mbi->mem_lower, mbi->mem_upper);
+    else if ( have_loader_memlimits(lctx) ) {
+        printk(TBOOT_DETA"no e820 map, mem_lower=%x, mem_upper=%x\n",
+               get_loader_mem_lower(lctx), get_loader_mem_upper(lctx));
 
         /* lower limit is 0x00000000 - <mem_lower>*0x400 (i.e. in kb) */
         g_copy_e820_map[0].base_addr_low = 0;
         g_copy_e820_map[0].base_addr_high = 0;
-        g_copy_e820_map[0].length_low = mbi->mem_lower << 10;
+        g_copy_e820_map[0].length_low = (get_loader_mem_lower(lctx)) << 10;
         g_copy_e820_map[0].length_high = 0;
         g_copy_e820_map[0].type = E820_RAM;
         g_copy_e820_map[0].size = sizeof(memory_map_t) - sizeof(uint32_t);
@@ -321,7 +344,7 @@ bool copy_e820_map(const multiboot_info_t *mbi)
         /* upper limit is 0x00100000 - <mem_upper>*0x400 */
         g_copy_e820_map[1].base_addr_low = 0x100000;
         g_copy_e820_map[1].base_addr_high = 0;
-        split64b((uint64_t)mbi->mem_upper << 10,
+        split64b((uint64_t)(get_loader_mem_upper(lctx)) << 10,
                  &(g_copy_e820_map[1].length_low),
                  &(g_copy_e820_map[1].length_high));
         g_copy_e820_map[1].type = E820_RAM;
@@ -330,21 +353,13 @@ bool copy_e820_map(const multiboot_info_t *mbi)
         g_nr_map = 2;
     }
     else {
-        printk("no e820 map nor memory limits provided\n");
+        printk(TBOOT_ERR"no e820 map nor memory limits provided\n");
         return false;
     }
 
     return true;
 }
 
-void replace_e820_map(multiboot_info_t *mbi)
-{
-    /* replace original with the copy */
-    mbi->mmap_addr = (uint32_t)g_copy_e820_map;
-    mbi->mmap_length = g_nr_map * sizeof(memory_map_t);
-    mbi->flags |= MBI_MEMMAP;   /* in case only MBI_MEMLIMITS was set */
-}
-
 bool e820_protect_region(uint64_t addr, uint64_t size, uint32_t type)
 {
     return protect_region(g_copy_e820_map, &g_nr_map, addr, size, type);
@@ -431,24 +446,24 @@ uint32_t e820_check_region(uint64_t base, uint64_t length)
         ret = E820_GAP;
 
     /* print the result */
-    printk(" (range from %016Lx to %016Lx is in ", base, base + length);
+    printk(TBOOT_DETA" (range from %016Lx to %016Lx is in ", base, base + length);
     switch (ret) {
         case E820_RAM:
-            printk("E820_RAM)\n"); break;
+            printk(TBOOT_INFO"E820_RAM)\n"); break;
         case E820_RESERVED:
-            printk("E820_RESERVED)\n"); break;
+            printk(TBOOT_INFO"E820_RESERVED)\n"); break;
         case E820_ACPI:
-            printk("E820_ACPI)\n"); break;
+            printk(TBOOT_INFO"E820_ACPI)\n"); break;
         case E820_NVS:
-            printk("E820_NVS)\n"); break;
+            printk(TBOOT_INFO"E820_NVS)\n"); break;
         case E820_UNUSABLE:
-            printk("E820_UNUSABLE)\n"); break;
+            printk(TBOOT_INFO"E820_UNUSABLE)\n"); break;
         case E820_GAP:
-            printk("E820_GAP)\n"); break;
+            printk(TBOOT_INFO"E820_GAP)\n"); break;
         case E820_MIXED:
-            printk("E820_MIXED)\n"); break;
+            printk(TBOOT_INFO"E820_MIXED)\n"); break;
         default:
-            printk("UNKNOWN)\n");
+            printk(TBOOT_INFO"UNKNOWN)\n");
     }
 
     return ret;
@@ -545,7 +560,7 @@ bool e820_reserve_ram(uint64_t base, uint64_t length)
             break;
         }
         else {
-            printk("we should never get here\n");
+            printk(TBOOT_ERR"we should never get here\n");
             return false;
         }
     }
@@ -579,9 +594,9 @@ bool get_ram_ranges(uint64_t *min_lo_ram, uint64_t *max_lo_ram,
      * to find it first so that we can tell when we have passed it
      */
     if ( g_min_ram > 0 ) {
-        get_highest_sized_ram(g_min_ram, 0x100000000ULL, &last_min_ram_base,
-                              &last_min_ram_size);
-        printk("highest min_ram (0x%x) region found: base=0x%Lx, size=0x%Lx\n",
+        e820_get_highest_sized_ram(g_min_ram, 0x100000000ULL,
+                                   &last_min_ram_base, &last_min_ram_size);
+        printk(TBOOT_DETA"highest min_ram (0x%x) region found: base=0x%Lx, size=0x%Lx\n",
                g_min_ram, last_min_ram_base, last_min_ram_size);
     }
 
@@ -593,7 +608,7 @@ bool get_ram_ranges(uint64_t *min_lo_ram, uint64_t *max_lo_ram,
         if ( entry->type == E820_RAM ) {
             /* if range straddles 4GB boundary, that is an error */
             if ( base < 0x100000000ULL && limit > 0x100000000ULL ) {
-                printk("e820 memory range straddles 4GB boundary\n");
+                printk(TBOOT_ERR"e820 memory range straddles 4GB boundary\n");
                 return false;
             }
 
@@ -611,9 +626,12 @@ bool get_ram_ranges(uint64_t *min_lo_ram, uint64_t *max_lo_ram,
             }
             else {     /* need to reserve low RAM above reserved regions */
                 if ( base < 0x100000000ULL ) {
-                    printk("discarding RAM above reserved regions: 0x%Lx - 0x%Lx\n", base, limit);
+                    printk(TBOOT_DETA"discarding RAM above reserved regions: 0x%Lx - 0x%Lx\n", base, limit);
                     if ( !e820_reserve_ram(base, limit - base) )
                         return false;
+                    if (!efi_memmap_reserve(base, limit - base)) {
+                        return false;
+                    }
                 }
             }
 
@@ -633,7 +651,7 @@ bool get_ram_ranges(uint64_t *min_lo_ram, uint64_t *max_lo_ram,
 
     /* no low RAM found */
     if ( *min_lo_ram >= *max_lo_ram ) {
-        printk("no low ram in e820 map\n");
+        printk(TBOOT_ERR"no low ram in e820 map\n");
         return false;
     }
     /* no high RAM found */
@@ -644,13 +662,13 @@ bool get_ram_ranges(uint64_t *min_lo_ram, uint64_t *max_lo_ram,
 }
 
 /* find highest (< <limit>) RAM region of at least <size> bytes */
-void get_highest_sized_ram(uint64_t size, uint64_t limit,
-                           uint64_t *ram_base, uint64_t *ram_size)
+bool e820_get_highest_sized_ram(uint64_t size, uint64_t limit,
+                                uint64_t *ram_base, uint64_t *ram_size)
 {
     uint64_t last_fit_base = 0, last_fit_size = 0;
 
     if ( ram_base == NULL || ram_size == NULL )
-        return;
+        return false;
 
     for ( unsigned int i = 0; i < g_nr_map; i++ ) {
         memory_map_t *entry = &g_copy_e820_map[i];
@@ -669,10 +687,16 @@ void get_highest_sized_ram(uint64_t size, uint64_t limit,
         }
     }
 
-    *ram_base = last_fit_base;
-    *ram_size = last_fit_size;
+    if (last_fit_size == 0) {
+        return false;
+    } else {
+        *ram_base = last_fit_base;
+        *ram_size = last_fit_size;
+        return true;
+    }
 }
 
+
 /*
  * Local variables:
  * mode: C
