@@ -45,9 +45,10 @@
 #include <tboot.h>
 #include <tb_policy.h>
 #include <tb_error.h>
-#include <vmac.h>
+#include <poly1305.h>
 #include <integrity.h>
 #include <tpm.h>
+#include <processor.h>
 
 #include <page.h>
 #include <paging.h>
@@ -80,7 +81,7 @@ extern void apply_policy(tb_error_t error);
 extern bool evtlog_append(uint8_t pcr, hash_list_t *hl, uint32_t type);
 
 typedef struct {
-    uint8_t mac_key[VMAC_KEY_LEN/8];
+    uint8_t mac_key[POLY1305_KEY_SIZE];
     uint8_t shared_key[sizeof(_tboot_shared.s3_key)];
 } sealed_secrets_t;
 
@@ -141,26 +142,26 @@ static bool seal_data(const void *data, size_t data_size, const void *secrets, s
 {
     /* TPM_Seal can only seal small data (like key or hash), so hash data */
     struct __packed {
-        tb_hash_t data_hash;
+        sha256_hash_t data_hash;
         uint8_t   secrets[secrets_size];
     } blob;
     uint32_t err;
     struct tpm_if *tpm = get_tpm();
     const struct tpm_if_fp *tpm_fp = get_tpm_fp();
 
-    memset(&blob, 0, sizeof(blob));
-    if ( !hash_buffer(data, data_size, &blob.data_hash, tpm->cur_alg) ) {
+    tb_memset(&blob, 0, sizeof(blob));
+    if ( !hash_buffer(data, data_size, (tb_hash_t *)&blob.data_hash, TB_HALG_SHA256) ) {
         printk(TBOOT_ERR"failed to hash data\n");
         return false;
     }
 
-    if ( secrets != NULL && secrets_size > 0 )  memcpy(blob.secrets, secrets, secrets_size);
+    if ( secrets != NULL && secrets_size > 0 )  tb_memcpy(blob.secrets, secrets, secrets_size);
 
     err = tpm_fp->seal(tpm, 2, sizeof(blob), (const uint8_t *)&blob, sealed_data_size, sealed_data);
     if ( !err )  printk(TBOOT_WARN"failed to seal data\n");
 
     /* since blob might contain secret, clear it */
-    memset(&blob, 0, sizeof(blob));
+    tb_memset(&blob, 0, sizeof(blob));
 
     return err;
 }
@@ -169,7 +170,7 @@ static bool verify_sealed_data(const uint8_t *sealed_data,  uint32_t sealed_data
 {
     /* sealed data is hash of state data and optional secret */
     struct __packed {
-        tb_hash_t data_hash;
+        sha256_hash_t data_hash;
         uint8_t   secrets[secrets_size];
     } blob;
     bool err = true;
@@ -186,26 +187,26 @@ static bool verify_sealed_data(const uint8_t *sealed_data,  uint32_t sealed_data
         goto done;
     }
 
-    /* verify that (hash of) current data maches sealed hash */
+    /* verify that (hash of) current data matches sealed hash */
     tb_hash_t curr_data_hash;
-    memset(&curr_data_hash, 0, sizeof(curr_data_hash));
-    if ( !hash_buffer(curr_data, curr_data_size, &curr_data_hash, tpm->cur_alg) ) {
+    tb_memset(&curr_data_hash, 0, sizeof(curr_data_hash));
+    if ( !hash_buffer(curr_data, curr_data_size, &curr_data_hash, TB_HALG_SHA256) ) {
         printk(TBOOT_WARN"failed to hash state data\n");
         goto done;
     }
-    if ( !are_hashes_equal(&blob.data_hash, &curr_data_hash, tpm->cur_alg) ) {
+    if ( !are_hashes_equal((tb_hash_t *)&blob.data_hash, &curr_data_hash, TB_HALG_SHA256) ) {
         printk(TBOOT_WARN"sealed hash does not match current hash\n");
         goto done;
     }
 
     if ( secrets != NULL && secrets_size > 0 )
-        memcpy(secrets, &blob.secrets, secrets_size);
+        tb_memcpy(secrets, &blob.secrets, secrets_size);
 
     err = false;
 
  done:
     /* clear secret from local memory */
-    memset(&blob, 0, sizeof(blob));
+    tb_memset(&blob, 0, sizeof(blob));
 
     return !err;
 }
@@ -220,7 +221,7 @@ bool seal_pre_k_state(void)
     const struct tpm_if_fp *tpm_fp = get_tpm_fp();
     
     /* save hash of current policy into g_pre_k_s3_state */
-    memset(&g_pre_k_s3_state.pol_hash, 0, sizeof(g_pre_k_s3_state.pol_hash));
+    tb_memset(&g_pre_k_s3_state.pol_hash, 0, sizeof(g_pre_k_s3_state.pol_hash));
     if ( !hash_policy(&g_pre_k_s3_state.pol_hash, tpm->cur_alg) ) {
         printk(TBOOT_ERR"failed to hash policy\n");
         goto error;
@@ -258,23 +259,24 @@ bool seal_pre_k_state(void)
     return false;
 }
 
-static bool measure_memory_integrity(vmac_t *mac, uint8_t key[VMAC_KEY_LEN/8])
+static bool measure_memory_integrity(uint8_t* mac, uint8_t* key)
 {
-    vmac_ctx_t ctx;
-    uint8_t nonce[16] = {};
+    POLY1305 ctx;
     unsigned long virt = MAC_VIRT_START;
 
 /* we require memory is 4K page aligned in tboot */
 #define MAC_ALIGN PAGE_SIZE
     COMPILE_TIME_ASSERT(MAC_VIRT_SIZE >= MAC_PAGE_SIZE);
     COMPILE_TIME_ASSERT((unsigned long)(-1) - MAC_VIRT_START > MAC_VIRT_SIZE );
-    COMPILE_TIME_ASSERT(PAGE_SIZE % VMAC_NHBYTES == 0);
+    COMPILE_TIME_ASSERT(PAGE_SIZE % POLY1305_BLOCK_SIZE == 0);
 
     /* enable paging */
     if ( !enable_paging() )
         return false;
 
-    vmac_set_key(key, &ctx);
+    sse_enable();
+
+    Poly1305_Init(&ctx, key);
     for ( unsigned int i = 0; i < _tboot_shared.num_mac_regions; i++ ) {
         uint64_t start = _tboot_shared.mac_regions[i].start;
 
@@ -349,22 +351,19 @@ static bool measure_memory_integrity(vmac_t *mac, uint8_t key[VMAC_KEY_LEN/8])
 
             /* MAC the 2-Mbyte pages */
             while ( (vend > vstart) && ((vend - vstart) >= MAC_PAGE_SIZE) ) {
-                vmac_update((uint8_t *)(uintptr_t)vstart, MAC_PAGE_SIZE, &ctx);
+                Poly1305_Update(&ctx, (uint8_t *)(uintptr_t)vstart, MAC_PAGE_SIZE);
                 vstart += MAC_PAGE_SIZE;
             }
             /* MAC the rest */
             if ( vend > vstart )
-                vmac_update((uint8_t *)(uintptr_t)vstart, vend - vstart, &ctx);
+                Poly1305_Update(&ctx, (uint8_t *)(uintptr_t)vstart, vend - vstart);
 
             /* destroy the mapping */
             if ( virt == MAC_VIRT_START )
                 destroy_tboot_mapping(MAC_VIRT_START, MAC_VIRT_END);
         } while ( start < end );
     }
-    *mac = vmac(NULL, 0, nonce, NULL, &ctx);
-
-    /* wipe ctx to ensure key not left in memory */
-    memset(&ctx, 0, sizeof(ctx));
+    Poly1305_Final(&ctx, mac);
 
     /* return to protected mode without paging */
     if (!disable_paging())
@@ -418,10 +417,10 @@ bool verify_integrity(void)
         goto error;
 
     /* Verify memory integrity against sealed value */
-    vmac_t mac;
-    if ( !measure_memory_integrity(&mac, secrets.mac_key) )
+    uint8_t mac[POLY1305_DIGEST_SIZE];
+    if ( !measure_memory_integrity(mac, secrets.mac_key) )
         goto error;
-    if ( memcmp(&mac, &g_post_k_s3_state.kernel_integ, sizeof(mac)) ) {
+    if ( tb_memcmp(&mac, &g_post_k_s3_state.kernel_integ, sizeof(mac)) ) {
         printk(TBOOT_INFO"memory integrity lost on S3 resume\n");
         printk(TBOOT_DETA"MAC of current image is: ");
         print_hex(NULL, &mac, sizeof(mac));
@@ -441,11 +440,11 @@ bool verify_integrity(void)
     }
 
     /* copy sealed shared key back to _tboot_shared.s3_key */
-    memcpy(_tboot_shared.s3_key, secrets.shared_key,
+    tb_memcpy(_tboot_shared.s3_key, secrets.shared_key,
            sizeof(_tboot_shared.s3_key));
 
     /* wipe secrets from memory */
-    memset(&secrets, 0, sizeof(secrets));
+    tb_memset(&secrets, 0, sizeof(secrets));
 
     return true;
 
@@ -480,10 +479,10 @@ bool seal_post_k_state(void)
     uint32_t key_size = sizeof(secrets.mac_key);
     /* key must be random and secret even though auth not necessary */
     if ( !tpm_fp->get_random(tpm, tpm->cur_loc, secrets.mac_key, &key_size) ||key_size != sizeof(secrets.mac_key) ) return false;
-    if ( !measure_memory_integrity(&g_post_k_s3_state.kernel_integ, secrets.mac_key) ) return false;
+    if ( !measure_memory_integrity(g_post_k_s3_state.kernel_integ, secrets.mac_key) ) return false;
 
     /* copy s3_key into secrets to be sealed */
-    memcpy(secrets.shared_key, _tboot_shared.s3_key, sizeof(secrets.shared_key));
+    tb_memcpy(secrets.shared_key, _tboot_shared.s3_key, sizeof(secrets.shared_key));
 
     print_post_k_s3_state();
 
@@ -491,7 +490,7 @@ bool seal_post_k_state(void)
     if ( !seal_data(&g_post_k_s3_state, sizeof(g_post_k_s3_state), &secrets, sizeof(secrets), sealed_post_k_state, &sealed_post_k_state_size) ) return false;
 
     /* wipe secrets from memory */
-    memset(&secrets, 0, sizeof(secrets));
+    tb_memset(&secrets, 0, sizeof(secrets));
 
     return true;
 }
