@@ -57,9 +57,10 @@
 #include <txt/acmod.h>
 #include <cmdline.h>
 #include <tpm.h>
+#include <efi_memmap.h>
 
 /* copy of kernel/VMM command line so that can append 'tboot=0x1234' */
-static char *new_cmdline = (char *)TBOOT_KERNEL_CMDLINE_ADDR;
+static char * volatile new_cmdline = (char *)TBOOT_KERNEL_CMDLINE_ADDR;
 
 /* MLE/kernel shared data page (in boot.S) */
 extern tboot_shared_t _tboot_shared;
@@ -94,11 +95,11 @@ printk_long(char *what)
 {
     /* chunk the command line into 70 byte chunks */
 #define CHUNK_SIZE 70
-    int      cmdlen = strlen(what);
+    int      cmdlen = tb_strlen(what);
     char    *cptr = what;
     char     cmdchunk[CHUNK_SIZE+1];
     while (cmdlen > 0) {
-        strncpy(cmdchunk, cptr, CHUNK_SIZE);
+        tb_strncpy(cmdchunk, cptr, CHUNK_SIZE);
         cmdchunk[CHUNK_SIZE] = 0;
         printk(TBOOT_INFO"\t%s\n", cmdchunk);
         cmdlen -= CHUNK_SIZE;
@@ -195,13 +196,13 @@ void print_mbi(const multiboot_info_t *mbi)
     if ( mbi->flags & MBI_CMDLINE ) {
 # define CHUNK_SIZE 72 
         /* Break the command line up into 72 byte chunks */
-        int   cmdlen = strlen(mbi->cmdline);
+        int   cmdlen = tb_strlen(mbi->cmdline);
         char *cmdptr = (char *)mbi->cmdline;
         char  chunk[CHUNK_SIZE+1];
         printk(TBOOT_DETA"\t cmdline@0x%x: ", mbi->cmdline);
         chunk[CHUNK_SIZE] = '\0';
         while (cmdlen > 0) {
-            strncpy(chunk, cmdptr, CHUNK_SIZE); 
+            tb_strncpy(chunk, cmdptr, CHUNK_SIZE); 
             printk(TBOOT_DETA"\n\t\"%s\"", chunk);
             cmdptr += CHUNK_SIZE;
             cmdlen -= CHUNK_SIZE;
@@ -288,8 +289,15 @@ bool verify_loader_context(loader_ctx *lctx)
     if (count < 1){
         printk(TBOOT_ERR"Error: no MB%d modules\n", lctx->type);
         return false;
-    } else
+    } else {
+        for (uint32_t i = 0; i < count; ++i) {
+            module_t *m = get_module(lctx, i);
+            if (m->mod_end < m->mod_start) {
+                return false;
+            }
+        }
         return true;
+    }
 }
 
 static bool remove_mb2_tag(loader_ctx *lctx, struct mb2_tag *cur)
@@ -355,6 +363,7 @@ grow_mb2_tag(loader_ctx *lctx, struct mb2_tag *which, uint32_t how_much)
     if (growth > slack){
         printk(TBOOT_ERR"YIKES!!! grow_mb2_tag slack %d < growth %d\n",
                slack, growth);
+        apply_policy(TB_ERR_FATAL);
     }
 
     /* now we copy down from the bottom, going up */
@@ -401,7 +410,7 @@ static void *remove_module(loader_ctx *lctx, void *mod_start)
         }
 
         /* copy remaing mods down by one */
-        memmove(m, m + 1, (mbi->mods_count - i - 1)*sizeof(module_t));
+        tb_memmove(m, m + 1, (mbi->mods_count - i - 1)*sizeof(module_t));
 
         mbi->mods_count--;
 
@@ -424,10 +433,10 @@ static void *remove_module(loader_ctx *lctx, void *mod_start)
                 printk(TBOOT_ERR"could not find module cmdline\n");
                 return NULL;
             }
-            if ((strlen(mod_string)) > (strlen(cmdline))){
-                if (strlen(mod_string) >= TBOOT_KERNEL_CMDLINE_SIZE){
+            if ((tb_strlen(mod_string)) > (tb_strlen(cmdline))){
+                if (tb_strlen(mod_string) >= TBOOT_KERNEL_CMDLINE_SIZE){
                     printk(TBOOT_ERR"No room to copy MB2 cmdline [%d < %d]\n",
-                           (int)(strlen(cmdline)), (int)(strlen(mod_string)));
+                           (int)(tb_strlen(cmdline)), (int)(tb_strlen(mod_string)));
                 } else {
                     char *s = mod_string;
                     char *d = cmdbuf;
@@ -504,7 +513,7 @@ static void *remove_module(loader_ctx *lctx, void *mod_start)
                 return NULL;
             }
 
-            grow_mb2_tag(lctx, cur, strlen(cmdbuf) - strlen(cmd->string));
+            grow_mb2_tag(lctx, cur, tb_strlen(cmdbuf) - tb_strlen(cmd->string));
 
             /* now we're all good, except for fixing up cmd */
             {
@@ -536,7 +545,7 @@ static bool adjust_kernel_cmdline(loader_ctx *lctx,
         if (old_cmdline == NULL)
             old_cmdline = "";
 
-        snprintf(new_cmdline, TBOOT_KERNEL_CMDLINE_SIZE, "%s tboot=%p",
+        tb_snprintf(new_cmdline, TBOOT_KERNEL_CMDLINE_SIZE, "%s tboot=%p",
                  old_cmdline, tboot_shared_addr);
         new_cmdline[TBOOT_KERNEL_CMDLINE_SIZE - 1] = '\0';
 
@@ -563,7 +572,7 @@ static bool adjust_kernel_cmdline(loader_ctx *lctx,
                 printk(TBOOT_ERR"adjust_kernel_cmdline() NULL MB2 cmd\n");
                 return NULL;
             }
-            uint32_t new_cmdline_tag_size = 2 * sizeof(uint32_t) + strlen(new_cmdline) + 1;
+            uint32_t new_cmdline_tag_size = 2 * sizeof(uint32_t) + tb_strlen(new_cmdline) + 1;
             if ( new_cmdline_tag_size > cmd->size ){
                 if (false ==
                     grow_mb2_tag(lctx, cur,
@@ -632,7 +641,7 @@ find_module(loader_ctx *lctx, void **base, size_t *size,
             printk(TBOOT_ERR"Error: image size is smaller than data size.\n");
             return false;
         }
-        if ( memcmp((void *)m->mod_start, data, len) == 0 ) {
+        if ( tb_memcmp((void *)m->mod_start, data, len) == 0 ) {
             *base = (void *)m->mod_start;
             if ( size != NULL )
                 *size = mod_size;
@@ -736,7 +745,7 @@ unsigned long get_mbi_mem_end_mb1(const multiboot_info_t *mbi)
     unsigned long end = (unsigned long)(mbi + 1);
 
     if ( mbi->flags & MBI_CMDLINE )
-        end = max(end, mbi->cmdline + strlen((char *)mbi->cmdline) + 1);
+        end = max(end, mbi->cmdline + tb_strlen((char *)mbi->cmdline) + 1);
     if ( mbi->flags & MBI_MODULES ) {
         end = max(end, mbi->mods_addr + mbi->mods_count * sizeof(module_t));
         unsigned int i;
@@ -744,7 +753,7 @@ unsigned long get_mbi_mem_end_mb1(const multiboot_info_t *mbi)
             module_t *p = get_module_mb1(mbi, i);
             if ( p == NULL )
                 break;
-            end = max(end, p->string + strlen((char *)p->string) + 1);
+            end = max(end, p->string + tb_strlen((char *)p->string) + 1);
         }
     }
     if ( mbi->flags & MBI_AOUT ) {
@@ -765,7 +774,7 @@ unsigned long get_mbi_mem_end_mb1(const multiboot_info_t *mbi)
     /*  GET CONFIGURATION bios call", so skip it */
     if ( mbi->flags & MBI_BTLDNAME )
         end = max(end, mbi->boot_loader_name
-                       + strlen((char *)mbi->boot_loader_name) + 1);
+                       + tb_strlen((char *)mbi->boot_loader_name) + 1);
     if ( mbi->flags & MBI_APM )
         /* per Grub-multiboot-Main Part2 Rev94-Structures, apm size is 20 */
         end = max(end, mbi->apm_table + 20);
@@ -785,7 +794,7 @@ unsigned long get_mbi_mem_end_mb1(const multiboot_info_t *mbi)
 static bool move_modules_to_high_memory(loader_ctx  *lctx)
 {
     uint32_t memRequired;
-    uint64_t max_ram_base, max_ram_size, ld_ceiling;
+    uint64_t max_ram_base = 0, max_ram_size = 0, ld_ceiling;
 
     uint32_t module_count, mod_i, mods_remaining;
     module_t *m;
@@ -800,16 +809,20 @@ static bool move_modules_to_high_memory(loader_ctx  *lctx)
         memRequired += PAGE_UP(m->mod_end - m->mod_start);
     }
 
-    /*  NOTE: the e820 map has been modified already to reserve critical
+    /*  NOTE: the memory maps have been modified already to reserve critical
         memory regions (tboot memory, etc ...). get_highest_sized_ram
         will return a range that excludes critical memory regions. */
-    get_highest_sized_ram(  memRequired, 0x100000000ULL,
-                            &max_ram_base, &max_ram_size);
-    if(!max_ram_base || !max_ram_size){
-        printk(TBOOT_INFO"ERROR No memory area found for image relocation!\n");
-        printk(TBOOT_INFO"required 0x%X\n", memRequired);
-        return false;
+    if (!efi_memmap_get_highest_sized_ram(memRequired, 0x100000000ULL,
+                                          &max_ram_base, &max_ram_size)) {
+        if (!e820_get_highest_sized_ram(memRequired, 0x100000000ULL,
+                                        &max_ram_base, &max_ram_size)) {
+            printk(TBOOT_INFO"ERROR No memory area found for image"
+                             "relocation!\n");
+            printk(TBOOT_INFO"required 0x%X\n", memRequired);
+            return false;
+        }
     }
+
     printk(TBOOT_INFO"highest suitable area @ 0x%llX (size 0x%llX)\n",
             max_ram_base, max_ram_size);
     ld_ceiling = PAGE_DOWN(max_ram_base + max_ram_size);
@@ -847,7 +860,7 @@ static bool move_modules_to_high_memory(loader_ctx  *lctx)
             uint32_t highest_mod_newbase = PAGE_DOWN(ld_ceiling-size);
             printk(TBOOT_INFO"moving module %u (%u B) from 0x%08X to 0x%08X\n",
                     highest_mod_i, size, highest_mod_base, highest_mod_newbase);
-            memcpy((void *)highest_mod_newbase, (void *)highest_mod_base, size);
+            tb_memcpy((void *)highest_mod_newbase, (void *)highest_mod_base, size);
             m->mod_start= highest_mod_newbase;
             m->mod_end  = highest_mod_newbase+size;
         }
@@ -929,7 +942,7 @@ static bool move_modules_above_elf_kernel(  loader_ctx      *lctx,
             uint32_t lowest_mod_newbase = ld_floor; /* already page aligned */
             printk(TBOOT_INFO"moving module %u (%u B) from 0x%08X to 0x%08X\n",
                     lowest_mod_i, size, lowest_mod_base, lowest_mod_newbase);
-            memcpy((void *)lowest_mod_newbase, (void *)lowest_mod_base, size);
+            tb_memcpy((void *)lowest_mod_newbase, (void *)lowest_mod_base, size);
             m->mod_start= lowest_mod_newbase;
             m->mod_end  = lowest_mod_newbase+size;
         }
@@ -1106,7 +1119,7 @@ move_modules(loader_ctx *lctx)
     if ( to < get_loader_ctx_end(lctx) )
         to = get_loader_ctx_end(lctx);
 
-    memcpy((void *)to, (void *)from, TBOOT_BASE_ADDR - from);
+    tb_memcpy((void *)to, (void *)from, TBOOT_BASE_ADDR - from);
     
     printk(TBOOT_DETA"0x%lx bytes copied from 0x%lx to 0x%lx\n",
            TBOOT_BASE_ADDR - from, from, to);
@@ -1285,7 +1298,7 @@ static bool convert_mb2_to_mb1(void)
             s = (uint8_t *)&apm->version;
             d = (uint8_t *) obd;  mbi->apm_table = obd;
             for (i = 0; 
-                 i < sizeof(struct mb2_tag_apm) - sizeof(uint32_t);
+                 i < sizeof(struct mb2_tag_apm) - 2*sizeof(uint32_t);
                  i++){
                 *d = *s;
                 d++; s++; obd++;
@@ -1383,12 +1396,18 @@ bool launch_kernel(bool is_measured_launch)
         printk(TBOOT_INFO"reserving tboot memory log (%Lx - %Lx) in e820 table\n", base, (base + size - 1));
         if ( !e820_protect_region(base, size, E820_RESERVED) )
             apply_policy(TB_ERR_FATAL);
+        if (!efi_memmap_reserve(base, size)) {
+            apply_policy(TB_ERR_FATAL);
+        }
     }
 
     /* replace map in loader context with copy */
     replace_e820_map(g_ldr_ctx);
-    printk(TBOOT_DETA"adjusted e820 map:\n");
-    print_e820_map();
+
+    if (get_tboot_dump_memmap()) {
+        printk(TBOOT_DETA"adjusted e820 map:\n");
+        print_e820_map();
+    }
 
     if ( !verify_loader_context(g_ldr_ctx) )
         return false;
@@ -1529,7 +1548,7 @@ find_module_by_file_signature(loader_ctx *lctx, void **base,
                               size_t *size, const char* file_signature)
 {
     return find_module(lctx, base, size, 
-                       file_signature, strlen(file_signature));
+                       file_signature, tb_strlen(file_signature));
 }
 
 bool 
@@ -1884,12 +1903,6 @@ replace_e820_map(loader_ctx *lctx)
                 old++, new++;
             }
         }
-        /* 
-           printk(TBOOT_INFO"AFTER replace_e820_map, loader context:\n");
-           print_loader_ctx(lctx);
-        */
-        printk(TBOOT_INFO"replaced memory map:\n");
-        print_e820_map();
         return;
     }
     return;
@@ -1902,7 +1915,8 @@ void print_loader_ctx(loader_ctx *lctx)
         return;
     } else {
         struct mb2_tag *start = (struct mb2_tag *)(lctx->addr + 8);
-        printk(TBOOT_INFO"MB2 dump, size %d\n", *(uint32_t *)lctx->addr);
+        printk(TBOOT_INFO"MB2 dump: addr %p, size %d\n", lctx->addr,
+               *(uint32_t *)lctx->addr);
         while (start != NULL){
             printk(TBOOT_INFO"MB2 tag found of type %d size %d ", 
                    start->type, start->size);
@@ -1917,8 +1931,10 @@ void print_loader_ctx(loader_ctx *lctx)
                 break;
             case MB2_TAG_TYPE_MODULE:
                 {
-                    struct mb2_tag_module *ts = 
+                    struct mb2_tag_module *ts =
                         (struct mb2_tag_module *) start;
+                    printk(TBOOT_INFO"mod_start: 0x%x, mod_end: 0x%x",
+                           ts->mod_start, ts->mod_end);
                     printk_long(ts->cmdline);
                 }
                 break;
@@ -1994,6 +2010,13 @@ find_efi_memmap(loader_ctx *lctx, uint32_t *descr_size,
     struct mb2_tag *start = NULL, *hit = NULL;
     struct mb2_tag_efi_mmap *efi_mmap = NULL;
 
+    if (LOADER_CTX_BAD(lctx) || lctx->type != MB2_ONLY) {
+        return 0;
+    }
+    if (descr_size == NULL || descr_vers == NULL || mmap_size == NULL) {
+        return 0;
+    }
+
     start = (struct mb2_tag *)(lctx->addr + 8);
     hit = find_mb2_tag_type(start, MB2_TAG_TYPE_EFI_MMAP);
     if (hit == NULL) {
@@ -2020,20 +2043,27 @@ is_loader_launch_efi(loader_ctx *lctx)
     return (get_loader_efi_ptr(lctx, &addr, &long_addr));
 }
 
-void load_framebuffer_info(loader_ctx *lctx, void *vscr)
+bool load_framebuffer_info(loader_ctx *lctx, void *vscr, bool efifb)
 {
     screen_info_t *scr = (screen_info_t *) vscr;
     struct mb2_tag *start;
 
     if (scr == NULL)
-        return;
-    if (LOADER_CTX_BAD(lctx))
-        return;
+        return false;
+    if (LOADER_CTX_BAD(lctx) || lctx->type != MB2_ONLY)
+        return false;
     start = (struct mb2_tag *)(lctx->addr + 8);
     start = find_mb2_tag_type(start, MB2_TAG_TYPE_FRAMEBUFFER);
     if (start != NULL){
+        u32 ext_lfb_base;
         struct mb2_fb *mbf = (struct mb2_fb *) start;
+
         scr->lfb_base = (uint32_t) mbf->common.fb_addr;
+        ext_lfb_base = (u64)mbf->common.fb_addr >> 32;
+        if (ext_lfb_base != 0) {
+            scr->capabilities |= VIDEO_CAPABILITY_64BIT_BASE;
+            scr->ext_lfb_base = ext_lfb_base;
+        }
         scr->lfb_width = mbf->common.fb_width;
         scr->lfb_height = mbf->common.fb_height;
         scr->lfb_depth =  mbf->common.fb_bpp;
@@ -2049,12 +2079,29 @@ void load_framebuffer_info(loader_ctx *lctx, void *vscr)
         /* round up to next 64k */
         scr->lfb_size = (scr->lfb_size + 65535) & 65535;
         
-        scr->orig_video_isVGA = 0x70; /* EFI FB */
+        if (efifb) {
+            scr->orig_video_isVGA = 0x70; /* EFI FB */
+        } else {
+            scr->orig_video_isVGA = 0x23; /* VESA VGA */
+        }
         scr->orig_y = 24;
+        return true;
+    } else {
+        return false;
     }
 
 }
 
+struct mb2_fb* get_framebuffer_info(loader_ctx *lctx)
+{
+    if (LOADER_CTX_BAD(lctx) || lctx->type != MB2_ONLY) {
+        return NULL;
+    }
+
+    struct mb2_tag *start = (struct mb2_tag *)(lctx->addr + 8);
+    return (struct mb2_fb*)find_mb2_tag_type(start, MB2_TAG_TYPE_FRAMEBUFFER);
+}
+
 void determine_loader_type(void *addr, uint32_t magic)
 {
     if (g_ldr_ctx->addr == NULL){
