@@ -54,7 +54,8 @@ __data struct tpm_if g_tpm = {
     .timeout.timeout_d = TIMEOUT_D,
 };
 
-u16 tboot_alg_list[] = {TB_HALG_SHA1, TB_HALG_SHA256};
+u16 tboot_alg_list[] = {TB_HALG_SHA1, TB_HALG_SHA256, TB_HALG_SHA384, TB_HALG_SHA512};
+const uint8_t tboot_alg_list_count = ARRAY_SIZE(tboot_alg_list);
 
 /* Global variables for TPM status register */
 static tpm20_reg_sts_t       g_reg_sts, *g_reg_sts_20 = &g_reg_sts;
@@ -101,7 +102,7 @@ else \
 static void tpm_send_cmd_ready_status(uint32_t locality)
 {
     /* write 1 to TPM_STS_x.commandReady to let TPM enter ready state */
-    memset((void *)&g_reg_sts, 0, sizeof(g_reg_sts));
+    tb_memset((void *)&g_reg_sts, 0, sizeof(g_reg_sts));
     g_reg_sts.command_ready = 1;
     write_tpm_sts_reg(locality);
 }
@@ -120,14 +121,14 @@ static bool tpm_send_cmd_ready_status_crb(uint32_t locality)
 #endif
 
 	if ( reg_ctrl_sts.tpmidle == 1) {
-           memset(&reg_ctrl_request,0,sizeof(reg_ctrl_request));
+           tb_memset(&reg_ctrl_request,0,sizeof(reg_ctrl_request));
            reg_ctrl_request.cmdReady = 1;
 	    write_tpm_reg(locality, TPM_CRB_CTRL_REQ, &reg_ctrl_request);
 
 	    return true;
 	}
 
-      memset(&reg_ctrl_request,0,sizeof(reg_ctrl_request));
+      tb_memset(&reg_ctrl_request,0,sizeof(reg_ctrl_request));
       reg_ctrl_request.goIdle = 1;
       write_tpm_reg(locality, TPM_CRB_CTRL_REQ, &reg_ctrl_request);
 	  
@@ -161,7 +162,7 @@ static bool tpm_send_cmd_ready_status_crb(uint32_t locality)
        printk(TBOOT_INFO"2. reg_ctrl_sts.tpmsts: 0x%x\n", reg_ctrl_sts.tpmsts); 	
 #endif
 
-       memset(&reg_ctrl_request,0,sizeof(reg_ctrl_request));
+       tb_memset(&reg_ctrl_request,0,sizeof(reg_ctrl_request));
        reg_ctrl_request.cmdReady = 1;
 	write_tpm_reg(locality, TPM_CRB_CTRL_REQ, &reg_ctrl_request);
 
@@ -252,7 +253,7 @@ static bool tpm_check_da_status(uint32_t locality)
 
 static void tpm_execute_cmd(uint32_t locality)
 {
-    memset((void *)&g_reg_sts, 0, sizeof(g_reg_sts));
+    tb_memset((void *)&g_reg_sts, 0, sizeof(g_reg_sts));
     g_reg_sts.tpm_go = 1;
     write_tpm_sts_reg(locality);
 }
@@ -727,7 +728,7 @@ bool tpm_relinquish_locality_crb(uint32_t locality)
     if ( reg_loc_state.loc_assigned == 0 )    return true;
 
     /* make inactive by writing a 1 */
-    memset(&reg_loc_ctrl,0,sizeof(reg_loc_ctrl));
+    tb_memset(&reg_loc_ctrl,0,sizeof(reg_loc_ctrl));
     reg_loc_ctrl.relinquish = 1;
     write_tpm_reg(locality, TPM_REG_LOC_CTRL, &reg_loc_ctrl);
 
@@ -768,11 +769,12 @@ bool prepare_tpm(void)
      * must ensure TPM_ACCESS_0.activeLocality bit is clear
      * (: locality is not active)
      */
-   if (is_tpm_crb()) 
-//   	return release_locality_crb(0);
-       return true;
-   else 
-   	return release_locality(0);
+   if ( is_tpm_crb() ) {
+       return tpm_relinquish_locality_crb(0);
+   }
+   else {
+       return release_locality(0);
+   }
 }
 
 bool tpm_request_locality_crb(uint32_t locality){
@@ -781,7 +783,7 @@ bool tpm_request_locality_crb(uint32_t locality){
     tpm_reg_loc_state_t  reg_loc_state;
     tpm_reg_loc_ctrl_t    reg_loc_ctrl;
     /* request access to the TPM from locality N */
-    memset(&reg_loc_ctrl,0,sizeof(reg_loc_ctrl));
+    tb_memset(&reg_loc_ctrl,0,sizeof(reg_loc_ctrl));
     reg_loc_ctrl.requestAccess = 1;
     write_tpm_reg(locality, TPM_REG_LOC_CTRL, &reg_loc_ctrl);
 
