@@ -244,7 +244,7 @@ static bool tpm12_pcr_read(struct tpm_if *ti, uint32_t locality,
 
     if ( out_size > sizeof(*out) )
         out_size = sizeof(*out);
-    memcpy((void *)out, WRAPPER_OUT_BUF, out_size);
+    tb_memcpy((void *)out, WRAPPER_OUT_BUF, out_size);
 
 #ifdef TPM_TRACE
     {
@@ -259,8 +259,7 @@ static bool tpm12_pcr_read(struct tpm_if *ti, uint32_t locality,
 static bool _tpm12_pcr_extend(struct tpm_if *ti, uint32_t locality,
                              uint32_t pcr, const tpm_digest_t* in)
 {
-    uint32_t ret, in_size = 0, out_size;
-    tpm12_digest_t * out = NULL;
+    uint32_t ret, in_size = 0, out_size = 0;
 
     if ( ti == NULL )
         return false;
@@ -269,15 +268,11 @@ static bool _tpm12_pcr_extend(struct tpm_if *ti, uint32_t locality,
         ti->error = TPM_BAD_PARAMETER;
         return false;
     }
-    if ( out == NULL )
-        out_size = 0;
-    else
-        out_size = sizeof(*out);
 
     /* copy pcr into buf in reversed byte order, then copy in data */
     reverse_copy(WRAPPER_IN_BUF, &pcr, sizeof(pcr));
     in_size += sizeof(pcr);
-    memcpy(WRAPPER_IN_BUF + in_size, (void *)in, sizeof(*(tpm12_digest_t *)in));
+    tb_memcpy(WRAPPER_IN_BUF + in_size, (void *)in, sizeof(*(tpm12_digest_t *)in));
     in_size += sizeof(*(tpm12_digest_t *)in);
 
     ret = tpm12_submit_cmd(locality, TPM_ORD_PCR_EXTEND, in_size, &out_size);
@@ -291,18 +286,6 @@ static bool _tpm12_pcr_extend(struct tpm_if *ti, uint32_t locality,
         return false;
     }
 
-    if ( out != NULL && out_size > 0 ) {
-       out_size = (out_size > sizeof(*out)) ? sizeof(*out) : out_size;
-       memcpy((void *)out, WRAPPER_OUT_BUF, out_size);
-    }
-
-#ifdef TPM_TRACE
-    {
-        printk(TBOOT_INFO"TPM: ");
-        print_hex(NULL, out->digest, out_size);
-    }
-#endif
-
     return true;
 }
 
@@ -353,7 +336,7 @@ static bool tpm12_pcr_reset(struct tpm_if *ti, uint32_t locality, uint32_t pcr)
     pcr_sel.pcr_select[pcr / 8] = 1 << (pcr % 8);
 
     in_size = sizeof(pcr_sel);
-    memcpy(WRAPPER_IN_BUF, (void *)&pcr_sel, in_size);
+    tb_memcpy(WRAPPER_IN_BUF, (void *)&pcr_sel, in_size);
 
     ret = tpm12_submit_cmd(locality, TPM_ORD_PCR_RESET, in_size, &out_size);
     if ( ret != TPM_SUCCESS ) {
@@ -421,7 +404,7 @@ static bool tpm12_nv_read_value(struct tpm_if *ti, uint32_t locality,
     reverse_copy(data_size, WRAPPER_OUT_BUF, sizeof(*data_size));
     *data_size = (*data_size > out_size) ? out_size : *data_size;
     if( *data_size > 0 )
-        memcpy(data, WRAPPER_OUT_BUF + sizeof(*data_size), *data_size);
+        tb_memcpy(data, WRAPPER_OUT_BUF + sizeof(*data_size), *data_size);
 
     return true;
 }
@@ -449,7 +432,7 @@ static bool tpm12_nv_write_value(struct tpm_if *ti, uint32_t locality,
     in_size += sizeof(offset);
     reverse_copy(WRAPPER_IN_BUF + in_size, &data_size, sizeof(data_size));
     in_size += sizeof(data_size);
-    memcpy(WRAPPER_IN_BUF + in_size, data, data_size);
+    tb_memcpy(WRAPPER_IN_BUF + in_size, data, data_size);
     in_size += data_size;
 
     ret = tpm12_submit_cmd(locality, TPM_ORD_NV_WRITE_VALUE,
@@ -575,7 +558,7 @@ typedef struct __packed {
 }
 
 #define UNLOAD_BLOB(buf, offset, blob, size) {\
-    memcpy(buf + offset, blob, size);\
+    tb_memcpy(buf + offset, blob, size);\
     offset += size;\
 }
 
@@ -626,7 +609,7 @@ typedef struct __packed {
 }
 
 #define LOAD_BLOB(buf, offset, blob, size) {\
-    memcpy(blob, buf + offset, size);\
+    tb_memcpy(blob, buf + offset, size);\
     offset += size;\
 }
 
@@ -1003,7 +986,7 @@ static uint32_t _tpm12_wrap_seal(uint32_t locality,
     UNLOAD_BLOB_TYPE(WRAPPER_IN_BUF, offset, &shared_secret);
     UNLOAD_BLOB_TYPE(WRAPPER_IN_BUF, offset, &nonce_even);
     sha1_buffer(WRAPPER_IN_BUF, offset, (uint8_t *)&digest);
-    memcpy(&enc_auth, &blob_authdata, sizeof(blob_authdata));
+    tb_memcpy(&enc_auth, &blob_authdata, sizeof(blob_authdata));
     XOR_BLOB_TYPE(&enc_auth, &digest);
 
     /* skip generate nonce for nonce_odd, just use the random value in stack */
@@ -1044,6 +1027,8 @@ static uint32_t _tpm12_wrap_seal(uint32_t locality,
 static uint32_t _tpm12_wrap_unseal(uint32_t locality, const uint8_t *in_data,
                                  uint32_t *secret_size, uint8_t *secret)
 {
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wmaybe-uninitialized"
     uint32_t ret;
     tpm_nonce_t odd_osap, even_osap;
     tpm_nonce_t nonce_even, nonce_odd, nonce_even_d, nonce_odd_d;
@@ -1116,6 +1101,7 @@ static uint32_t _tpm12_wrap_unseal(uint32_t locality, const uint8_t *in_data,
     /* skip check for res_auth */
 
     return ret;
+#pragma GCC diagnostic pop
 }
 
 static bool init_pcr_info(uint32_t locality,
@@ -1150,7 +1136,7 @@ static bool init_pcr_info(uint32_t locality,
             return false;
     }
 
-    memset(pcr_info, 0, sizeof(*pcr_info));
+    tb_memset(pcr_info, 0, sizeof(*pcr_info));
     pcr_info->tag = TPM_TAG_PCR_INFO_LONG;
     pcr_info->locality_at_creation = localities[locality];
     pcr_info->locality_at_release = release_locs;
@@ -1340,7 +1326,7 @@ static bool tpm12_cmp_creation_pcrs(uint32_t pcr_nr_create,
     cre_composite = get_cre_pcr_composite(sealed_data);
     if ( cre_composite == NULL )
         return false;
-    if ( memcmp(&composite, cre_composite, sizeof(composite)) ) {
+    if ( tb_memcmp(&composite, cre_composite, sizeof(composite)) ) {
         printk(TBOOT_WARN"TPM: Not equal to creation composition:\n");
         print_hex(NULL, (uint8_t *)&composite, sizeof(composite));
         print_hex(NULL, (uint8_t *)cre_composite, sizeof(composite));
@@ -1733,6 +1719,8 @@ static bool tpm12_init(struct tpm_if *ti)
     if ( ti == NULL )
         return false;
 
+    printk(TBOOT_WARN"Warning: TPM1.2 detected, SHA1 is selected as hashing algorithm.\n");
+
     if (!txt_is_launched())
         ti->cur_loc = 0;
     else
@@ -1745,7 +1733,7 @@ static bool tpm12_init(struct tpm_if *ti)
     }
 
     /* make sure tpm is not disabled/deactivated */
-    memset(&pflags, 0, sizeof(pflags));
+    tb_memset(&pflags, 0, sizeof(pflags));
     ret = tpm12_get_flags(locality, TPM_CAP_FLAG_PERMANENT,
                         (uint8_t *)&pflags, sizeof(pflags));
     if ( ret != TPM_SUCCESS ) {
@@ -1758,7 +1746,7 @@ static bool tpm12_init(struct tpm_if *ti)
         return false;
     }
 
-    memset(&vflags, 0, sizeof(vflags));
+    tb_memset(&vflags, 0, sizeof(vflags));
     ret = tpm12_get_flags(locality, TPM_CAP_FLAG_VOLATILE,
                         (uint8_t *)&vflags, sizeof(vflags));
     if ( ret != TPM_SUCCESS ) {
@@ -1921,10 +1909,10 @@ static bool tpm12_get_random(struct tpm_if *ti, uint32_t locality,
         return false;
     }
     if ( *data_size > 0 )
-        memcpy(random_data, WRAPPER_OUT_BUF + sizeof(*data_size), *data_size);
+        tb_memcpy(random_data, WRAPPER_OUT_BUF + sizeof(*data_size), *data_size);
 
     /* data might be used as key, so clear from buffer memory */
-    memset(WRAPPER_OUT_BUF + sizeof(*data_size), 0, *data_size);
+    tb_memset(WRAPPER_OUT_BUF + sizeof(*data_size), 0, *data_size);
 
     /* if TPM doesn't return all requested random bytes, try one more time */
     if ( *data_size < requested_size ) {
@@ -1948,6 +1936,8 @@ static bool tpm12_get_random(struct tpm_if *ti, uint32_t locality,
 
 static bool tpm12_cap_pcrs(struct tpm_if *ti, u32 locality, int pcr)
 {
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wmaybe-uninitialized"
     bool was_capped[TPM_NR_PCRS] = {false};
     tpm_pcr_value_t cap_val;   /* use whatever val is on stack */
 
@@ -1966,7 +1956,7 @@ static bool tpm12_cap_pcrs(struct tpm_if *ti, u32 locality, int pcr)
 
     /* also cap every dynamic PCR we extended (only once) */
     /* don't cap static PCRs since then they would be wrong after S3 resume */
-    memset(&was_capped, true, TPM_PCR_RESETABLE_MIN*sizeof(bool));
+    tb_memset(&was_capped, true, TPM_PCR_RESETABLE_MIN*sizeof(bool));
     for ( int i = 0; i < g_pre_k_s3_state.num_vl_entries; i++ ) {
         if ( !was_capped[g_pre_k_s3_state.vl_entries[i].pcr] ) {
             _tpm12_pcr_extend(ti, locality, g_pre_k_s3_state.vl_entries[i].pcr, &cap_val);
@@ -1976,6 +1966,7 @@ static bool tpm12_cap_pcrs(struct tpm_if *ti, u32 locality, int pcr)
 
     printk(TBOOT_INFO"cap'ed dynamic PCRs\n");
     return true;
+#pragma GCC diagnostic pop
 }
 
 static bool tpm12_check(void)
