@@ -44,8 +44,6 @@
 #include <string.h>
 #include <tpm.h>
 #include <tpm_20.h>
-#include <sha1.h>
-#include <sha256.h>
 #include <cmdline.h>
 #include <uuid.h>
 #include <mle.h>
@@ -173,7 +171,7 @@ static u16 reverse_copy_sized_buf_in(TPM2B *dest, TPM2B *src)
 
 /*
  * Inputs: dest->size should contain the buffer size of dest->buffer[]
- * Ouputs: dest->size should contain the final copied data size
+ * Outputs: dest->size should contain the final copied data size
  *
  * Return: 0, failed; 2+, succeed.
  */
@@ -1430,7 +1428,7 @@ static uint32_t _tpm20_nv_read(uint32_t locality,
     cmd_size = (u8 *)other - cmd_buf;
     reverse_copy(cmd_buf + CMD_SIZE_OFFSET, &cmd_size, sizeof(cmd_size));
 
-    rsp_size = sizeof(*out);
+    rsp_size = sizeof(rsp_buf);
  
     if (g_tpm_family == TPM_IF_20_FIFO) {
         if (!tpm_submit_cmd(locality, cmd_buf, cmd_size, rsp_buf, &rsp_size))
@@ -1799,8 +1797,8 @@ static uint32_t _tpm20_create(uint32_t locality,
     cmd_size = (u8 *)other - cmd_buf;
     reverse_copy(cmd_buf + CMD_SIZE_OFFSET, &cmd_size, sizeof(cmd_size));
 
-    rsp_size = sizeof(*out);
-    
+    rsp_size = sizeof(rsp_buf);
+
     if (g_tpm_family == TPM_IF_20_FIFO) {
         if (!tpm_submit_cmd(locality, cmd_buf, cmd_size, rsp_buf, &rsp_size))
             return TPM_RC_FAILURE;
@@ -2240,7 +2238,7 @@ static bool tpm20_hash(struct tpm_if *ti, u32 locality, const u8 *data,
         }
 
         buffer.t.size = chunk_size;
-        memcpy( &(buffer.t.buffer[0]), &(data[i] ), chunk_size );
+        tb_memcpy( &(buffer.t.buffer[0]), &(data[i] ), chunk_size );
 
         update_in.buf = buffer;
         ret = _tpm20_sequence_update(locality, &update_in, &update_out);
@@ -2269,7 +2267,7 @@ static bool tpm20_hash(struct tpm_if *ti, u32 locality, const u8 *data,
 
     for ( j=0; j<hl->count; j++ ) {
         hl->entries[j].alg = complete_out.results.digests[j].hash_alg;
-        memcpy(&hl->entries[j].hash, &complete_out.results.digests[j].digest,
+        tb_memcpy(&hl->entries[j].hash, &complete_out.results.digests[j].digest,
                 sizeof(hl->entries[j].hash));
     }
 
@@ -2311,7 +2309,7 @@ static bool tpm20_nv_read(struct tpm_if *ti, uint32_t locality,
         return false;
     }
     *data_size = read_out.data.t.size;
-    memcpy(data, &read_out.data.t.buffer[0], *data_size);
+    tb_memcpy(data, &read_out.data.t.buffer[0], *data_size);
 
     return true;
 }
@@ -2334,7 +2332,7 @@ static bool tpm20_nv_write(struct tpm_if *ti, uint32_t locality,
     write_in.sessions.sessions[0] = pw_session;
     write_in.offset = offset;
     write_in.data.t.size = data_size;
-    memcpy(&write_in.data.t.buffer[0], data, data_size);
+    tb_memcpy(&write_in.data.t.buffer[0], data, data_size);
 
     ret = _tpm20_nv_write(locality, &write_in, &write_out);
     if ( ret != TPM_RC_SUCCESS ) {
@@ -2381,10 +2379,32 @@ static bool tpm20_get_nvindex_size(struct tpm_if *ti, uint32_t locality,
 static bool tpm20_get_nvindex_permission(struct tpm_if *ti, uint32_t locality,
                                     uint32_t index, uint32_t *attribute)
 {
+    tpm_nv_read_public_in public_in;
+    tpm_nv_read_public_out public_out;
+    u32 ret;
+
     if ( ti == NULL || locality >= TPM_NR_LOCALITIES
          || index == 0 || attribute == NULL )
         return false;
 
+    public_in.index = index;
+
+    ret = _tpm20_nv_read_public(locality, &public_in, &public_out);
+    if ( ret != TPM_RC_SUCCESS ) {
+        printk(TBOOT_WARN"TPM: fail to get public data of 0x%08X in TPM NV\n", index);
+        ti->error = ret;
+        return false;
+    }
+
+    if (index != public_out.nv_public.t.nv_public.index) {
+        printk(TBOOT_WARN"TPM: Index 0x%08X is not the one expected 0x%08X\n",
+                index, index);
+        ti->error = TPM_RC_FAILURE;
+        return false;
+    }
+
+    *attribute = *(uint32_t*)(&public_out.nv_public.t.nv_public.attr);
+
     return true;
 }
 
@@ -2412,16 +2432,26 @@ static bool tpm20_seal(struct tpm_if *ti, uint32_t locality,
     create_in.public.t.public_area.param.keyed_hash.scheme.scheme = TPM_ALG_NULL;
     create_in.public.t.public_area.unique.keyed_hash.t.size = 0;
 
+    COMPILE_TIME_ASSERT( sizeof(auth_str) - 1 <=
+            sizeof(create_in.sensitive.t.sensitive.user_auth.t.buffer) );
     create_in.sensitive.t.sensitive.user_auth.t.size = sizeof(auth_str) - 1;
-    memcpy(&(create_in.sensitive.t.sensitive.user_auth.t.buffer[0]),
+    tb_memcpy(&(create_in.sensitive.t.sensitive.user_auth.t.buffer[0]),
             auth_str, sizeof(auth_str)-1);
+    if ( in_data_size >
+            sizeof(create_in.sensitive.t.sensitive.data.t.buffer) ) {
+        printk(TBOOT_WARN"TPM: input data size to seal is too large:"
+               " %08X(%08x)\n",
+               in_data_size,
+               sizeof(create_in.sensitive.t.sensitive.data.t.buffer));
+        return false;
+    }
     create_in.sensitive.t.sensitive.data.t.size = in_data_size;
-    memcpy(&(create_in.sensitive.t.sensitive.data.t.buffer[0]),
+    tb_memcpy(&(create_in.sensitive.t.sensitive.data.t.buffer[0]),
             in_data, in_data_size); 
 
     create_in.outside_info.t.size = 0;
     create_in.creation_pcr.count = 0;
-    memset(&create_out, 0, sizeof(create_out));
+    tb_memset(&create_out, 0, sizeof(create_out));
 
     ret = _tpm20_create(locality, &create_in, &create_out);
     if ( ret != TPM_RC_SUCCESS ) {
@@ -2430,7 +2460,7 @@ static bool tpm20_seal(struct tpm_if *ti, uint32_t locality,
         return false;
     }
     *sealed_data_size = sizeof(create_out);
-    memcpy(sealed_data, &create_out, *sealed_data_size); 
+    tb_memcpy(sealed_data, &create_out, *sealed_data_size); 
 
     return true;
 }
@@ -2469,7 +2499,7 @@ static bool tpm20_unseal(struct tpm_if *ti, uint32_t locality,
     unseal_in.sessions.num_sessions = 1;
     unseal_in.sessions.sessions[0] = pw_session;
     unseal_in.sessions.sessions[0].hmac.t.size = sizeof(auth_str) - 1;
-    memcpy(&(unseal_in.sessions.sessions[0].hmac.t.buffer[0]),
+    tb_memcpy(&(unseal_in.sessions.sessions[0].hmac.t.buffer[0]),
             auth_str, sizeof(auth_str)-1);
     unseal_in.item_handle = load_out.obj_handle;
 
@@ -2481,7 +2511,7 @@ static bool tpm20_unseal(struct tpm_if *ti, uint32_t locality,
     }
 
     *secret_size = unseal_out.data.t.size;
-    memcpy(secret, &(unseal_out.data.t.buffer[0]), *secret_size);
+    tb_memcpy(secret, &(unseal_out.data.t.buffer[0]), *secret_size);
 
     return true;
 }
@@ -2519,7 +2549,7 @@ static bool tpm20_get_random(struct tpm_if *ti, uint32_t locality,
 
     out_size = random_out.random_bytes.t.size;
     if (out_size > 0)
-        memcpy(random_data, &(random_out.random_bytes.t.buffer[0]), out_size);
+        tb_memcpy(random_data, &(random_out.random_bytes.t.buffer[0]), out_size);
     *data_size = out_size;
 
     /* if TPM doesn't return all requested random bytes, try one more time */
@@ -2542,7 +2572,7 @@ static bool tpm20_get_random(struct tpm_if *ti, uint32_t locality,
 
             out_size = random_out.random_bytes.t.size;
             if (out_size > 0)
-                memcpy(random_data+*data_size, 
+                tb_memcpy(random_data+*data_size, 
                         &(random_out.random_bytes.t.buffer[0]), out_size);
             *data_size += out_size;
         }
@@ -2593,7 +2623,7 @@ static bool tpm20_cap_pcrs(struct tpm_if *ti, u32 locality, int pcr)
 
     /* also cap every dynamic PCR we extended (only once) */
     /* don't cap static PCRs since then they would be wrong after S3 resume */
-    memset(&was_capped, true, TPM_PCR_RESETABLE_MIN*sizeof(bool));
+    tb_memset(&was_capped, true, TPM_PCR_RESETABLE_MIN*sizeof(bool));
     for ( int i = 0; i < g_pre_k_s3_state.num_vl_entries; i++ ) {
         if ( !was_capped[g_pre_k_s3_state.vl_entries[i].pcr] ) {
             tpm20_pcr_extend(ti, locality, g_pre_k_s3_state.vl_entries[i].pcr, &cap_val);
@@ -2607,7 +2637,7 @@ static bool tpm20_cap_pcrs(struct tpm_if *ti, u32 locality, int pcr)
 
 static bool alg_is_supported(u16 alg)
 {
-    for (int i=0; i<2; i++) {
+    for (int i = 0; i < tboot_alg_list_count; i++) {
         if (alg == tboot_alg_list[i])
             return true;
     }
@@ -2636,7 +2666,7 @@ static bool tpm20_context_save(struct tpm_if *ti, u32 locality, TPM_HANDLE handl
     }
     else 
 	 printk(TBOOT_WARN"TPM: tpm2 context save successful, return value = %08X\n", ret);
-    memcpy((tpm_contextsave_out *)context_saved, &out, sizeof(tpm_contextsave_out));
+    tb_memcpy((tpm_contextsave_out *)context_saved, &out, sizeof(tpm_contextsave_out));
     return true;
 }	
 
@@ -2649,7 +2679,7 @@ static bool tpm20_context_load(struct tpm_if *ti, u32 locality, void  *context_s
     if ( ti == NULL || locality >= TPM_NR_LOCALITIES )
         return false;
 	
-    memcpy(&in, (tpm_contextsave_out *)context_saved, sizeof(tpm_contextsave_out));
+    tb_memcpy(&in, (tpm_contextsave_out *)context_saved, sizeof(tpm_contextsave_out));
 
     ret = _tpm20_context_load(locality, &in, &out);
     if ( ret != TPM_RC_SUCCESS ) {
