@@ -521,7 +521,44 @@ bool set_mem_type(const void *base, uint32_t size, uint32_t mem_type)
 
     printk(TBOOT_DETA"setting MTRRs for acmod: base=%p, size=%x, num_pages=%d\n",
            base, size, num_pages);
+    /*
+     * Each VAR MTRR base must be a multiple if that MTRR's Size
+    */
+    unsigned long base_v;
+    base_v = (unsigned long) base;
+    int i =0;   
+    // mtrr size in pages
+    int mtrr_s = 1;
+    while ((base_v & 0x01) == 0) {
+          i++;
+          base_v = base_v >>1 ;
 
+    }
+    for (int j=i-12; j>0; j--) mtrr_s =mtrr_s*2; //mtrr_s = mtrr_s << 1
+    printk(TBOOT_DETA"The maximum allowed MTRR range size=%d Pages \n", mtrr_s);
+	
+    while (num_pages >= mtrr_s){
+	
+	/* set the base of the current MTRR */
+        mtrr_physbase.raw = rdmsr(MTRR_PHYS_BASE0_MSR + ndx*2);
+        mtrr_physbase.base = ((unsigned long)base >> PAGE_SHIFT) &
+	                     SINIT_MTRR_MASK;
+        mtrr_physbase.type = mem_type;
+        wrmsr(MTRR_PHYS_BASE0_MSR + ndx*2, mtrr_physbase.raw);
+
+        mtrr_physmask.raw = rdmsr(MTRR_PHYS_MASK0_MSR + ndx*2);
+        mtrr_physmask.mask = ~(mtrr_s - 1) & SINIT_MTRR_MASK;
+        mtrr_physmask.v = 1;
+        wrmsr(MTRR_PHYS_MASK0_MSR + ndx*2, mtrr_physmask.raw);
+		
+        base += (mtrr_s * PAGE_SIZE);
+        num_pages -= mtrr_s;
+        ndx++;
+        if ( ndx == mtrr_cap.vcnt ) {
+            printk(TBOOT_ERR"exceeded number of var MTRRs when mapping range\n");
+            return false;
+        }
+    }
     while ( num_pages > 0 ) {
         uint32_t pages_in_range;
 
@@ -544,7 +581,7 @@ bool set_mem_type(const void *base, uint32_t size, uint32_t mem_type)
         mtrr_physmask.v = 1;
         wrmsr(MTRR_PHYS_MASK0_MSR + ndx*2, mtrr_physmask.raw);
 
-        /* prepare for the next loop depending on number of pages
+        /*prepare for the next loop depending on number of pages
          * We figure out from the above how many pages could be used in this
          * mtrr. Then we decrement the count, increment the base,
          * increment the mtrr we are dealing with, and if num_pages is
@@ -558,7 +595,6 @@ bool set_mem_type(const void *base, uint32_t size, uint32_t mem_type)
             return false;
         }
     }
-
     return true;
 }
 
